from fastapi import APIRouter
from subprocess import check_output

from src.constants import GAME_VERSION, SERVER_VERSION
from src.errors import Response


api = APIRouter(prefix='/api', tags=['Service'])


@api.get('/version')
async def get_versions():
    commit_hash = git_revision_hash()
    git_server = {
        'commit': git_revision_short_hash(),
        'commit_hash': commit_hash,
        'branch': git_branch(),
        'release': git_text(commit_hash)
    }

    data_hash = git_revision_hash('./data')
    git_data = {
        'commit': git_revision_short_hash('./data'),
        'commit_hash': data_hash,
        'branch': git_branch('./data'),
        'release': git_text(data_hash, './data')
    }
    reponse = {'server_version': SERVER_VERSION, 
               'game_version': GAME_VERSION,
               'git_server': git_server,
               'git_data': git_data}
    return Response(response=reponse)


def git_revision_short_hash(cwd=None) -> str:
    return check_out(['git', 'rev-parse', '--short', 'HEAD'], cwd)


def git_revision_hash(cwd=None) -> str:
    return check_out(['git', 'rev-parse', 'HEAD'], cwd)


def git_branch(cwd=None) -> str:
    return check_out(['git', 'branch', '--show-current'], cwd)


def git_text(commit, cwd=None) -> str:
    return check_out(['git', 'log', '--format=%B', '-n', '1', commit], cwd)


def check_out(args, cwd=None):
    if not cwd:
        cwd = './'
    return check_output(args, cwd=cwd).decode('ascii').strip()
